@@ -31,7 +31,6 @@
 #ifndef LIBXSMM_FRONTEND_H
 #define LIBXSMM_FRONTEND_H
 
-#include "libxsmm_typedefs.h"
 #include "libxsmm_macros.h"
 #include "libxsmm.h"
 
@@ -49,6 +48,7 @@
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
 #endif
+#include "libxsmm_generator.h"
 #include <assert.h>
 #include <stdint.h>
 #if defined(LIBXSMM_OFFLOAD_TARGET)
@@ -77,15 +77,16 @@
 #endif
 
 /** Helper macros for eliding prefetch address calculations depending on prefetch scheme. */
-#if 0 != ((LIBXSMM_PREFETCH) & 2) || 0 != ((LIBXSMM_PREFETCH) & 4)
+#if 0 != ((LIBXSMM_PREFETCH) & 2/*AL2*/) || 0 != ((LIBXSMM_PREFETCH) & 4/*AL2_JPST*/)
 # define LIBXSMM_PREFETCH_A(EXPR) (EXPR)
 #endif
-#if 0 != ((LIBXSMM_PREFETCH) & 8)
+#if 0 != ((LIBXSMM_PREFETCH) & 8/*BL2_VIA_C*/)
 # define LIBXSMM_PREFETCH_B(EXPR) (EXPR)
 #endif
 #if 0/*no scheme yet using C*/
 # define LIBXSMM_PREFETCH_C(EXPR) (EXPR)
 #endif
+/** Secondary helper macros derived from the above group. */
 #if defined(LIBXSMM_PREFETCH_A)
 # define LIBXSMM_NOPREFETCH_A(EXPR)
 #else
@@ -105,11 +106,11 @@
 # define LIBXSMM_PREFETCH_C(EXPR) 0
 #endif
 
-/** Helper macro for GEMM function names (and similar functions). */
-#define LIBXSMM_TPREFIX_AUX(REAL, FUNCTION) LIBXSMM_TPREFIX_##REAL(FUNCTION)
-#define LIBXSMM_TPREFIX(REAL, FUNCTION) LIBXSMM_TPREFIX_AUX(REAL, FUNCTION)
-#define LIBXSMM_TPREFIX_double(FUNCTION) d##FUNCTION
-#define LIBXSMM_TPREFIX_float(FUNCTION) s##FUNCTION
+/** Helper macro for BLAS-style symbol names. */
+#define LIBXSMM_TPREFIX_NAME(REAL) LIBXSMM_CONCATENATE(LIBXSMM_TPREFIX_, REAL)
+#define LIBXSMM_TPREFIX(REAL, SYMBOL) LIBXSMM_CONCATENATE(LIBXSMM_TPREFIX_NAME(REAL), SYMBOL)
+#define LIBXSMM_TPREFIX_double d
+#define LIBXSMM_TPREFIX_float s
 
 /** Check ILP64 configuration for sanity. */
 #if (defined(MKL_ILP64) && 0 == LIBXSMM_ILP64)
@@ -151,14 +152,27 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void (*libxsmm_internal_dgemm)(
 #endif
 
 /** Construct symbol name from a given real type name (float or double). */
-#define LIBXSMM_BLAS_GEMM_SYMBOL(REAL)  LIBXSMM_CONCATENATE(libxsmm_internal_, LIBXSMM_TPREFIX(REAL, gemm))
-#define LIBXSMM_XBLAS_GEMM_SYMBOL(REAL) LIBXSMM_CONCATENATE(libxsmm_blas_, LIBXSMM_TPREFIX(REAL, gemm))
 #define LIBXSMM_MMFUNCTION_TYPE(REAL)   LIBXSMM_CONCATENATE(libxsmm_, LIBXSMM_TPREFIX(REAL, mmfunction))
 #define LIBXSMM_MMDISPATCH_SYMBOL(REAL) LIBXSMM_CONCATENATE(libxsmm_, LIBXSMM_TPREFIX(REAL, mmdispatch))
+#define LIBXSMM_BLAS_GEMM_SYMBOL(REAL)  LIBXSMM_CONCATENATE(libxsmm_internal_, LIBXSMM_TPREFIX(REAL, gemm))
+#define LIBXSMM_XBLAS_SYMBOL(REAL)      LIBXSMM_CONCATENATE(libxsmm_blas_, LIBXSMM_TPREFIX(REAL, gemm))
+#define LIBXSMM_XOMPS_SYMBOL(REAL)      LIBXSMM_CONCATENATE(libxsmm_omps_, LIBXSMM_TPREFIX(REAL, gemm))
 #define LIBXSMM_XGEMM_SYMBOL(REAL)      LIBXSMM_CONCATENATE(libxsmm_, LIBXSMM_TPREFIX(REAL, gemm))
 
+/** Helper macro consolidating the applicable GEMM arguments into LIBXSMM's flags. */
+#define LIBXSMM_GEMM_DECLARE_FLAGS(FLAGS, TRANSA, TRANSB, M, N, K, A, B, C) \
+  int FLAGS = (0 != (TRANSA) \
+    ? (('N' == *(TRANSA) || 'n' == *(TRANSA)) ? (LIBXSMM_FLAGS & ~LIBXSMM_GEMM_FLAG_TRANS_A) \
+                                              : (LIBXSMM_FLAGS |  LIBXSMM_GEMM_FLAG_TRANS_A)) \
+    : LIBXSMM_FLAGS); \
+  FLAGS = (0 != (TRANSB) \
+    ? (('N' == *(TRANSB) || 'n' == *(TRANSB)) ? ((FLAGS) & ~LIBXSMM_GEMM_FLAG_TRANS_B) \
+                                              : ((FLAGS) |  LIBXSMM_GEMM_FLAG_TRANS_B)) \
+    : (FLAGS)); \
+  assert(0 != (M) && 0 != (N) && 0 != (K) && 0 != (A) && 0 != (B) && 0 != (C))
+
 /** BLAS-based GEMM supplied by the linked LAPACK/BLAS library (template). */
-#define LIBXSMM_BLAS_XGEMM(REAL, SYMBOL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) { \
+#define LIBXSMM_BLAS_XGEMM(REAL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) { \
   const char libxsmm_blas_xgemm_transa_ = (char)(0 == (LIBXSMM_GEMM_FLAG_TRANS_A & (FLAGS)) ? 'N' : 'T'); \
   const char libxsmm_blas_xgemm_transb_ = (char)(0 == (LIBXSMM_GEMM_FLAG_TRANS_B & (FLAGS)) ? 'N' : 'T'); \
   const REAL libxsmm_blas_xgemm_alpha_ = (REAL)(ALPHA), libxsmm_blas_xgemm_beta_ = (REAL)(BETA); \
@@ -168,11 +182,8 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void (*libxsmm_internal_dgemm)(
   const libxsmm_blasint libxsmm_blas_xgemm_m_ = (libxsmm_blasint)LIBXSMM_LD(M, N); \
   const libxsmm_blasint libxsmm_blas_xgemm_n_ = (libxsmm_blasint)LIBXSMM_LD(N, M); \
   const libxsmm_blasint libxsmm_blas_xgemm_k_ = (libxsmm_blasint)(K); \
-  assert(libxsmm_blas_xgemm_m_ <= libxsmm_blas_xgemm_lda_); \
-  assert(libxsmm_blas_xgemm_k_ <= libxsmm_blas_xgemm_ldb_); \
-  assert(libxsmm_blas_xgemm_m_ <= libxsmm_blas_xgemm_ldc_); \
-  assert(0 != ((uintptr_t)SYMBOL)); \
-  SYMBOL(&libxsmm_blas_xgemm_transa_, &libxsmm_blas_xgemm_transb_, \
+  assert(0 != ((uintptr_t)LIBXSMM_BLAS_GEMM_SYMBOL(REAL))); \
+  LIBXSMM_BLAS_GEMM_SYMBOL(REAL)(&libxsmm_blas_xgemm_transa_, &libxsmm_blas_xgemm_transb_, \
     &libxsmm_blas_xgemm_m_, &libxsmm_blas_xgemm_n_, &libxsmm_blas_xgemm_k_, \
     &libxsmm_blas_xgemm_alpha_, (const REAL*)LIBXSMM_LD(A, B), &libxsmm_blas_xgemm_lda_, \
                                 (const REAL*)LIBXSMM_LD(B, A), &libxsmm_blas_xgemm_ldb_, \
@@ -181,10 +192,10 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void (*libxsmm_internal_dgemm)(
 
 /** BLAS-based GEMM supplied by the linked LAPACK/BLAS library (single-precision). */
 #define LIBXSMM_BLAS_SGEMM(FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
-  LIBXSMM_BLAS_XGEMM(float, libxsmm_internal_sgemm, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
+  LIBXSMM_BLAS_XGEMM(float, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
 /** BLAS-based GEMM supplied by the linked LAPACK/BLAS library (single-precision). */
 #define LIBXSMM_BLAS_DGEMM(FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
-  LIBXSMM_BLAS_XGEMM(double, libxsmm_internal_dgemm, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
+  LIBXSMM_BLAS_XGEMM(double, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
 /** BLAS-based GEMM supplied by the linked LAPACK/BLAS library. */
 #define LIBXSMM_BLAS_GEMM(FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) { \
   if (sizeof(double) == sizeof(*(A))) { \
@@ -197,18 +208,16 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void (*libxsmm_internal_dgemm)(
 
 /** Inlinable GEMM exercising the compiler's code generation (template). */
 #if defined(MKL_DIRECT_CALL_SEQ) || defined(MKL_DIRECT_CALL)
-# define LIBXSMM_INLINE_XGEMM(REAL, INT, SYMBOL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
-  assert(0 != ((uintptr_t)SYMBOL)); \
-  LIBXSMM_BLAS_XGEMM(REAL, SYMBOL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
+# define LIBXSMM_INLINE_XGEMM(REAL, INT, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
+  LIBXSMM_BLAS_XGEMM(REAL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
 #else
-# define LIBXSMM_INLINE_XGEMM(REAL, INT, SYMBOL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) { \
+# define LIBXSMM_INLINE_XGEMM(REAL, INT, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) { \
   const REAL libxsmm_inline_xgemm_alpha_ = (REAL)(1 == (ALPHA) ? 1 : (-1 == (ALPHA) ? -1 : (ALPHA))); \
   const REAL libxsmm_inline_xgemm_beta_ = (REAL)(1 == (BETA) ? 1 : (0 == (BETA) ? 0 : (BETA))); \
   INT libxsmm_inline_xgemm_i_, libxsmm_inline_xgemm_j_, libxsmm_inline_xgemm_k_; \
   assert(0 == (LIBXSMM_GEMM_FLAG_TRANS_A & (FLAGS)) && 0 == (LIBXSMM_GEMM_FLAG_TRANS_B & (FLAGS))/*not supported*/); \
-  assert(LIBXSMM_LD(M, N) <= LIBXSMM_LD(LDA, LDB)); \
-  assert((K) <= LIBXSMM_LD(LDB, LDA)); \
-  assert(LIBXSMM_LD(M, N) <= (LDC)); \
+  /* TODO: remove/adjust precondition if anything other than NN is supported */ \
+  assert(LIBXSMM_LD(M, N) <= LIBXSMM_LD(LDA, LDB) && (K) <= LIBXSMM_LD(LDB, LDA) && LIBXSMM_LD(M, N) <= (LDC)); \
   LIBXSMM_PRAGMA_SIMD \
   for (libxsmm_inline_xgemm_j_ = 0; libxsmm_inline_xgemm_j_ < ((INT)LIBXSMM_LD(M, N)); ++libxsmm_inline_xgemm_j_) { \
     LIBXSMM_PRAGMA_LOOP_COUNT(1, LIBXSMM_MAX_K, LIBXSMM_AVG_K) \
@@ -227,10 +236,10 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void (*libxsmm_internal_dgemm)(
 
 /** Inlinable GEMM exercising the compiler's code generation (single-precision). */
 #define LIBXSMM_INLINE_SGEMM(FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
-  LIBXSMM_INLINE_XGEMM(float, libxsmm_blasint, libxsmm_internal_sgemm, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
+  LIBXSMM_INLINE_XGEMM(float, libxsmm_blasint, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
 /** Inlinable GEMM exercising the compiler's code generation (double-precision). */
 #define LIBXSMM_INLINE_DGEMM(FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
-  LIBXSMM_INLINE_XGEMM(double, libxsmm_blasint, libxsmm_internal_dgemm, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
+  LIBXSMM_INLINE_XGEMM(double, libxsmm_blasint, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
 /** Inlinable GEMM exercising the compiler's code generation. */
 #define LIBXSMM_INLINE_GEMM(FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) { \
   if (sizeof(double) == sizeof(*(A))) { \
@@ -241,16 +250,30 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void (*libxsmm_internal_dgemm)(
   } \
 }
 
+#define LIBXSMM_OMPS_GEMM(REAL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
+  LIBXSMM_XOMPS_SYMBOL(REAL)( \
+    0 == (LIBXSMM_GEMM_FLAG_TRANS_A & (FLAGS)) ? 'N' : 'T', \
+    0 == (LIBXSMM_GEMM_FLAG_TRANS_B & (FLAGS)) ? 'N' : 'T', \
+    M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
+
 /** Fallback code paths: LIBXSMM_FALLBACK0, and LIBXSMM_FALLBACK1 (template). */
 #if defined(LIBXSMM_FALLBACK_INLINE_GEMM)
-# define LIBXSMM_FALLBACK0(REAL, INT, SYMBOL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
-    LIBXSMM_INLINE_XGEMM(REAL, INT, SYMBOL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
+# define LIBXSMM_FALLBACK0(REAL, INT, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
+    LIBXSMM_INLINE_XGEMM(REAL, INT, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
+#elif defined(LIBXSMM_FALLBACK_OMPS)
+# define LIBXSMM_FALLBACK0(REAL, INT, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
+    LIBXSMM_OMPS_GEMM(REAL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
 #else
-# define LIBXSMM_FALLBACK0(REAL, INT, SYMBOL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
-    LIBXSMM_BLAS_XGEMM(REAL, SYMBOL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
+# define LIBXSMM_FALLBACK0(REAL, INT, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
+    LIBXSMM_BLAS_XGEMM(REAL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
+#endif
+#if defined(LIBXSMM_FALLBACK_OMPS)
+# define LIBXSMM_FALLBACK1(REAL, INT, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
+    LIBXSMM_OMPS_GEMM(REAL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
+#else
+# define LIBXSMM_FALLBACK1(REAL, INT, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
+    LIBXSMM_BLAS_XGEMM(REAL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
 #endif
-#define LIBXSMM_FALLBACK1(REAL, INT, SYMBOL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
-  LIBXSMM_BLAS_XGEMM(REAL, SYMBOL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
 
 /** Helper macros for calling a dispatched function in a row/column-major aware fashion. */
 #define LIBXSMM_MMCALL_ABC(FN, A, B, C) FN(LIBXSMM_LD(A, B), LIBXSMM_LD(B, A), C)
@@ -264,12 +287,20 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void (*libxsmm_internal_dgemm)(
     LIBXSMM_PREFETCH_C(PC)); \
 }
 
+#if (0/*LIBXSMM_PREFETCH_NONE*/ == LIBXSMM_PREFETCH)
+# define LIBXSMM_MMCALL(FN, A, B, C, M, N, K, LDA, LDB, LDC) \
+  LIBXSMM_MMCALL_ABC(FN, A, B, C)
+#else
+# define LIBXSMM_MMCALL(FN, A, B, C, M, N, K, LDA, LDB, LDC) \
+  LIBXSMM_MMCALL_PRF(FN, A, B, C, (A) + (LDA) * (K), (B) + (LDB) * (N), (C) + (LDC) * (N))
+#endif
+
 /**
  * Execute a specialized function, or use a fallback code path depending on threshold (template).
  * LIBXSMM_FALLBACK0 or specialized function: below LIBXSMM_MAX_MNK
  * LIBXSMM_FALLBACK1: above LIBXSMM_MAX_MNK
  */
-#define LIBXSMM_XGEMM(REAL, INT, SYMBOL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) { \
+#define LIBXSMM_XGEMM(REAL, INT, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) { \
   if (((unsigned long long)(LIBXSMM_MAX_MNK)) >= \
      (((unsigned long long)(M)) * \
       ((unsigned long long)(N)) * \
@@ -278,51 +309,27 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void (*libxsmm_internal_dgemm)(
     const int libxsmm_xgemm_flags_ = (int)(FLAGS); \
     const int libxsmm_xgemm_lda_ = (int)(LDA), libxsmm_xgemm_ldb_ = (int)(LDB), libxsmm_xgemm_ldc_ = (int)(LDC); \
     const REAL libxsmm_xgemm_alpha_ = (REAL)(ALPHA), libxsmm_xgemm_beta_ = (REAL)(BETA); \
-    int libxsmm_xgemm_fallback_ = 0; \
-    if (LIBXSMM_PREFETCH_NONE == LIBXSMM_PREFETCH) { \
-      const LIBXSMM_MMFUNCTION_TYPE(REAL) libxsmm_mmfunction_ = \
-        LIBXSMM_MMDISPATCH_SYMBOL(REAL)((int)(M), (int)(N), (int)(K), \
-          &libxsmm_xgemm_lda_, &libxsmm_xgemm_ldb_, &libxsmm_xgemm_ldc_, \
-          &libxsmm_xgemm_alpha_, &libxsmm_xgemm_beta_, \
-          &libxsmm_xgemm_flags_, 0); \
-      if (0 != libxsmm_mmfunction_) { \
-        LIBXSMM_MMCALL_ABC(libxsmm_mmfunction_, (const REAL*)(A), (const REAL*)(B), (REAL*)(C)); \
-      } \
-      else { \
-        libxsmm_xgemm_fallback_ = 1; \
-      } \
+    const LIBXSMM_MMFUNCTION_TYPE(REAL) libxsmm_mmfunction_ = LIBXSMM_MMDISPATCH_SYMBOL(REAL)( \
+      (int)(M), (int)(N), (int)(K), &libxsmm_xgemm_lda_, &libxsmm_xgemm_ldb_, &libxsmm_xgemm_ldc_, \
+      &libxsmm_xgemm_alpha_, &libxsmm_xgemm_beta_, &libxsmm_xgemm_flags_, 0); \
+    if (0 != libxsmm_mmfunction_) { \
+      LIBXSMM_MMCALL(libxsmm_mmfunction_, (const REAL*)(A), (const REAL*)(B), (REAL*)(C), M, N, K, LDA, LDB, LDC); \
     } \
     else { \
-      const int libxsmm_xgemm_prefetch_ = (LIBXSMM_PREFETCH); \
-      const LIBXSMM_MMFUNCTION_TYPE(REAL) libxsmm_mmfunction_ = \
-        LIBXSMM_MMDISPATCH_SYMBOL(REAL)((int)(M), (int)(N), (int)(K), \
-          &libxsmm_xgemm_lda_, &libxsmm_xgemm_ldb_, &libxsmm_xgemm_ldc_, \
-          &libxsmm_xgemm_alpha_, &libxsmm_xgemm_beta_, \
-          &libxsmm_xgemm_flags_, &libxsmm_xgemm_prefetch_); \
-      if (0 != libxsmm_mmfunction_) { \
-        LIBXSMM_MMCALL_PRF(libxsmm_mmfunction_, (const REAL*)(A), (const REAL*)(B), (REAL*)(C), \
-          ((const REAL*)(A)) + libxsmm_xgemm_lda_ * (K), ((const REAL*)(B)) + libxsmm_xgemm_ldb_ * (N), \
-          ((const REAL*)(C)) + libxsmm_xgemm_ldc_ * (N)); \
-      } \
-      else { \
-        libxsmm_xgemm_fallback_ = 1; \
-      } \
-    } \
-    if (0 != libxsmm_xgemm_fallback_) { \
-      LIBXSMM_FALLBACK0(REAL, INT, SYMBOL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC); \
+      LIBXSMM_FALLBACK0(REAL, INT, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC); \
     } \
   } \
   else { \
-    LIBXSMM_FALLBACK1(REAL, INT, SYMBOL, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC); \
+    LIBXSMM_FALLBACK1(REAL, INT, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC); \
   } \
 }
 
 /** Dispatched general dense matrix multiplication (single-precision). */
 #define LIBXSMM_SGEMM(FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
-  LIBXSMM_XGEMM(float, libxsmm_blasint, libxsmm_internal_sgemm, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
+  LIBXSMM_XGEMM(float, libxsmm_blasint, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
 /** Dispatched general dense matrix multiplication (double-precision). */
 #define LIBXSMM_DGEMM(FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) \
-  LIBXSMM_XGEMM(double, libxsmm_blasint, libxsmm_internal_dgemm, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
+  LIBXSMM_XGEMM(double, libxsmm_blasint, FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
 /** Dispatched general dense matrix multiplication. */
 #define LIBXSMM_GEMM(FLAGS, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC) { \
   if (sizeof(double) == sizeof(*(A))) { \
