@@ -132,24 +132,41 @@
 # define LIBXSMM_PRAGMA_UNROLL
 #endif
 
-/*Based on Stackoverflow's NBITSx macro.*/
-#define LIBXSMM_NBITS02(N) (0 != ((N) & 2/*0b10*/) ? 1 : 0)
-#define LIBXSMM_NBITS04(N) (0 != ((N) & 0xC/*0b1100*/) ? (2 + LIBXSMM_NBITS02((N) >> 2)) : LIBXSMM_NBITS02(N))
-#define LIBXSMM_NBITS08(N) (0 != ((N) & 0xF0/*0b11110000*/) ? (4 + LIBXSMM_NBITS04((N) >> 4)) : LIBXSMM_NBITS04(N))
-#define LIBXSMM_NBITS16(N) (0 != ((N) & 0xFF00) ? (8 + LIBXSMM_NBITS08((N) >> 8)) : LIBXSMM_NBITS08(N))
-#define LIBXSMM_NBITS32(N) (0 != ((N) & 0xFFFF0000) ? (16 + LIBXSMM_NBITS16((N) >> 16)) : LIBXSMM_NBITS16(N))
-#define LIBXSMM_NBITS64(N) (0 != ((N) & 0xFFFFFFFF00000000) ? (32 + LIBXSMM_NBITS32((N) >> 32)) : LIBXSMM_NBITS32(N))
-#define LIBXSMM_NBITS(N) (0 != (N) ? (LIBXSMM_NBITS64((unsigned long long)(N)) + 1) : 1)
+#if defined(_OPENMP) && (200805 <= _OPENMP) /*OpenMP 3.0*/
+# define LIBXSMM_OPENMP_COLLAPSE(N) collapse(N)
+#else
+# define LIBXSMM_OPENMP_COLLAPSE(N)
+#endif
+
+#define LIBXSMM_REPEAT_1(A) A
+#define LIBXSMM_REPEAT_2(A) LIBXSMM_REPEAT_1(A); A
+#define LIBXSMM_REPEAT_3(A) LIBXSMM_REPEAT_2(A); A
+#define LIBXSMM_REPEAT_4(A) LIBXSMM_REPEAT_3(A); A
+#define LIBXSMM_REPEAT_5(A) LIBXSMM_REPEAT_4(A); A
+#define LIBXSMM_REPEAT_6(A) LIBXSMM_REPEAT_5(A); A
+#define LIBXSMM_REPEAT_7(A) LIBXSMM_REPEAT_6(A); A
+#define LIBXSMM_REPEAT_8(A) LIBXSMM_REPEAT_7(A); A
+#define LIBXSMM_REPEAT(N, A) LIBXSMM_CONCATENATE(LIBXSMM_REPEAT_, N)(A)
 
 #define LIBXSMM_DEFAULT(DEFAULT, VALUE) (0 < (VALUE) ? (VALUE) : (DEFAULT))
+#define LIBXSMM_ABS(A) (0 <= (A) ? (A) : -(A))
 #define LIBXSMM_MIN(A, B) ((A) < (B) ? (A) : (B))
 #define LIBXSMM_MAX(A, B) ((A) < (B) ? (B) : (A))
 #define LIBXSMM_MOD2(N, NPOT) ((N) & ((NPOT) - 1))
-#define LIBXSMM_MUL2(N, NPOT) ((N) << (LIBXSMM_NBITS(NPOT) - 1))
-#define LIBXSMM_DIV2(N, NPOT) ((N) >> (LIBXSMM_NBITS(NPOT) - 1))
+#define LIBXSMM_MUL2(N, NPOT) ((N) << LIBXSMM_LOG2(NPOT))
+#define LIBXSMM_DIV2(N, NPOT) ((N) >> LIBXSMM_LOG2(NPOT))
 #define LIBXSMM_UP2(N, NPOT) LIBXSMM_MUL2(LIBXSMM_DIV2((N) + (NPOT) - 1, NPOT), NPOT)
 #define LIBXSMM_UP(N, UP) ((((N) + (UP) - 1) / (UP)) * (UP))
 
+/*Based on Stackoverflow's NBITSx macro.*/
+#define LIBXSMM_LOG2_02(N) (0 != ((N) & 2/*0b10*/) ? 1 : 0)
+#define LIBXSMM_LOG2_04(N) (0 != ((N) & 0xC/*0b1100*/) ? (2 + LIBXSMM_LOG2_02((N) >> 2)) : LIBXSMM_LOG2_02(N))
+#define LIBXSMM_LOG2_08(N) (0 != ((N) & 0xF0/*0b11110000*/) ? (4 + LIBXSMM_LOG2_04((N) >> 4)) : LIBXSMM_LOG2_04(N))
+#define LIBXSMM_LOG2_16(N) (0 != ((N) & 0xFF00) ? (8 + LIBXSMM_LOG2_08((N) >> 8)) : LIBXSMM_LOG2_08(N))
+#define LIBXSMM_LOG2_32(N) (0 != ((N) & 0xFFFF0000) ? (16 + LIBXSMM_LOG2_16((N) >> 16)) : LIBXSMM_LOG2_16(N))
+#define LIBXSMM_LOG2_64(N) (0 != ((N) & 0xFFFFFFFF00000000) ? (32 + LIBXSMM_LOG2_32((N) >> 32)) : LIBXSMM_LOG2_32(N))
+#define LIBXSMM_LOG2(N) LIBXSMM_MAX(LIBXSMM_LOG2_64((unsigned long long)(N)), 1)
+
 #if defined(_WIN32) && !defined(__GNUC__)
 # define LIBXSMM_ATTRIBUTE(...) __declspec(__VA_ARGS__)
 # if defined(__cplusplus)
@@ -213,7 +230,7 @@
 #endif
 
 #if defined(_REENTRANT)
-# if (defined(_WIN32) && !defined(__GNUC__)) || defined(__clang__)
+# if (defined(_WIN32) && !defined(__GNUC__))
 #   define LIBXSMM_TLS LIBXSMM_ATTRIBUTE(thread)
 # elif defined(__GNUC__)
 #   define LIBXSMM_TLS __thread
@@ -243,20 +260,20 @@
 
 /** Execute the CPUID, and receive results (EAX, EBX, ECX, EDX) for requested FUNCTION. */
 #if defined(__GNUC__)
-# define LIBXSMM_CPUID(FUNCTION, EAX, EBX, ECX, EDX) \
+# define LIBXSMM_CPUID_X86(FUNCTION, EAX, EBX, ECX, EDX) \
     __asm__ __volatile__ ("cpuid" : "=a"(EAX), "=b"(EBX), "=c"(ECX), "=d"(EDX) : "a"(FUNCTION))
 #else
-# define LIBXSMM_CPUID(FUNCTION, EAX, EBX, ECX, EDX) { \
-    int libxsmm_cpuid_[4]; \
-    __cpuid(libxsmm_cpuid_, FUNCTION); \
-    EAX = libxsmm_cpuid_[0]; \
-    EBX = libxsmm_cpuid_[1]; \
-    ECX = libxsmm_cpuid_[2]; \
-    EDX = libxsmm_cpuid_[3]; \
+# define LIBXSMM_CPUID_X86(FUNCTION, EAX, EBX, ECX, EDX) { \
+    int libxsmm_cpuid_x86_[4]; \
+    __cpuid(libxsmm_cpuid_x86_, FUNCTION); \
+    EAX = libxsmm_cpuid_x86_[0]; \
+    EBX = libxsmm_cpuid_x86_[1]; \
+    ECX = libxsmm_cpuid_x86_[2]; \
+    EDX = libxsmm_cpuid_x86_[3]; \
   }
 #endif
 
-/** Execute the XGETBV, and receive results (EAX, EDX) for req. eXtended Control Register (XCR). */
+/** Execute the XGETBV (x86), and receive results (EAX, EDX) for req. eXtended Control Register (XCR). */
 #if defined(__GNUC__)
 # define LIBXSMM_XGETBV(XCR, EAX, EDX) __asm__ __volatile__( \
     ".byte 0x0f, 0x01, 0xd0" /*xgetbv*/ : "=a"(EAX), "=d"(EDX) : "c"(XCR) \
