@@ -41,7 +41,7 @@
 #include <cassert>
 #include <cstdio>
 #include <cmath>
-#if defined(USE_MKL) || defined(MKL_DIRECT_CALL_SEQ) || defined(MKL_DIRECT_CALL)
+#if defined(__MKL) || defined(MKL_DIRECT_CALL_SEQ) || defined(MKL_DIRECT_CALL)
 # include <mkl_service.h>
 #endif
 #if defined(_OPENMP)
