@@ -52,8 +52,8 @@ if __name__ == "__main__":
                 print("  " + mnksig + ", " + ldxsig + ",")
                 print("  LIBXSMM_ALPHA, LIBXSMM_BETA, LIBXSMM_PREFETCH);")
                 print("LIBXSMM_HASH_FUNCTION_CALL(hash, indx, LIBXSMM_HASH_FUNCTION, desc);")
-                print("if (0 == result[indx].code.xmm) { /* no further effort to handle collision */")
-                print("  result[indx].code.smm = (libxsmm_smmfunction)libxsmm_smm_" + mnkstr + ";")
+                print("if (0 == result[indx].code.pmm) { /* no further effort to handle collision */")
+                print("  result[indx].code.xmm.smm = (libxsmm_smmfunction)libxsmm_smm_" + mnkstr + ";")
                 print("  result[indx].code_size = 0; /* statically generated code */")
                 print("  result[indx].descriptor = desc;")
                 print("} LIBXSMM_DEBUG(else ++cdp;)")
@@ -62,8 +62,8 @@ if __name__ == "__main__":
                 print("  " + mnksig + ", " + ldxsig + ",")
                 print("  LIBXSMM_ALPHA, LIBXSMM_BETA, LIBXSMM_PREFETCH);")
                 print("LIBXSMM_HASH_FUNCTION_CALL(hash, indx, LIBXSMM_HASH_FUNCTION, desc);")
-                print("if (0 == result[indx].code.xmm) { /* no further effort to handle collision */")
-                print("  result[indx].code.dmm = (libxsmm_dmmfunction)libxsmm_dmm_" + mnkstr + ";")
+                print("if (0 == result[indx].code.pmm) { /* no further effort to handle collision */")
+                print("  result[indx].code.xmm.dmm = (libxsmm_dmmfunction)libxsmm_dmm_" + mnkstr + ";")
                 print("  result[indx].code_size = 0; /* statically generated code */")
                 print("  result[indx].descriptor = desc;")
                 print("} LIBXSMM_DEBUG(else ++csp;)")
