@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 ###############################################################################
-## Copyright (c) 2013-2015, Intel Corporation                                ##
+## Copyright (c) 2015-2016, Intel Corporation                                ##
 ## All rights reserved.                                                      ##
 ##                                                                           ##
 ## Redistribution and use in source and binary forms, with or without        ##
@@ -29,10 +29,15 @@
 ###############################################################################
 ## Hans Pabst (Intel Corp.)
 ###############################################################################
-from functools import reduce
 import itertools
 import operator
 import sys, os
+import re
+
+try:
+    from functools import reduce
+except:
+    pass
 
 
 def upper_list(lists, level):
@@ -47,12 +52,24 @@ def upper_list(lists, level):
         return []
 
 
+# https://docs.python.org/3/library/itertools.html#itertools.product
+def itertools_product(*args):
+    # product('ABCD', 'xy') --> Ax Ay Bx By Cx Cy Dx Dy
+    # product(range(2), repeat=3) --> 000 001 010 011 100 101 110 111
+    pools = [tuple(pool) for pool in args]
+    result = [[]]
+    for pool in pools:
+        result = [x+[y] for x in result for y in pool]
+    for prod in result:
+        yield tuple(prod)
+
+
 def load_mnklist(argv, threshold, format = 0, resultset = set()):
     if (0 == format): # indexes format
         resultset = set(map(lambda mnk: tuple(map(int, mnk.split("_"))), argv))
     elif (-1 == format): # new input format
         groups = map(lambda group: [int(i) for i in group.split()], " ".join(argv[0:]).split(","))
-        resultset = set(itertools.chain(*[list(itertools.product(*(i, i, i))) for i in groups]))
+        resultset = set(itertools.chain(*[list(itertools_product(*(i, i, i))) for i in groups]))
     elif (-2 == format): # legacy format
         mlist = list(map(int, map(lambda s: str(s).replace(",", " ").strip(), argv[2:2+int(argv[0])])))
         nlist = list(map(int, map(lambda s: str(s).replace(",", " ").strip(), argv[2+int(argv[0]):2+int(argv[0])+int(argv[1])])))
@@ -73,13 +90,15 @@ def load_mnklist(argv, threshold, format = 0, resultset = set()):
     else:
         sys.tracebacklimit = 0
         raise ValueError("load_mnklist: unexpected format!")
-    return set(sorted(filter(lambda mnk: (0 < mnk[0]) and (0 < mnk[1]) and (0 < mnk[2]), resultset)))
+    return set(filter(lambda mnk: (0 < mnk[0]) and (0 < mnk[1]) and (0 < mnk[2]), resultset))
 
 
 def max_mnk(mnklist, init = 0, index = None):
-    return reduce(max, map(lambda mnk: \
-      mnk[index] if (None != index and 0 <= index and index < 3) \
-      else (mnk[0] * mnk[1] * mnk[2]), mnklist), init)
+    if (None != index and 0 <= index and index < 3):
+        mapped = map(lambda mnk: mnk[index], mnklist)
+    else:
+        mapped = map(lambda mnk: mnk[0] * mnk[1] * mnk[2], mnklist)
+    return reduce(max, mapped, init)
 
 
 def median(list_of_numbers, fallback = None, average = True):
@@ -92,12 +111,16 @@ def median(list_of_numbers, fallback = None, average = True):
             medval = int(0.5 * (list_of_numbers[size2-1] + list_of_numbers[size2]) + 0.5)
         else:
             medval = list_of_numbers[size2]
-        return min(medval, fallback) if None != fallback else medval
+        if (None != fallback):
+            result = min(medval, fallback)
+        else:
+            result = medval
     elif (None != fallback):
-        return fallback
+        result = fallback
     else:
         sys.tracebacklimit = 0
         raise ValueError("median: empty list!")
+    return result
 
 
 def is_pot(num):
@@ -124,23 +147,31 @@ def align_value(n, typesize, alignment):
 def version_branch():
     versionfile = os.path.join(os.path.sep, os.path.dirname(sys.argv[0]), "..", "version.txt")
     version = "1.0"
-    with open(versionfile, "r") as file:
+    file = open(versionfile, "r")
+    try:
         version = file.read().replace("\n", "")
-    versionlist = version.split("-")
-    return ("-".join(map(str, versionlist[1:])), versionlist[0]) if (2 < len(versionlist)) else (version, "")
+        versionlist = version.split("-")
+        if (1 < len(versionlist)):
+            result = ("-".join(map(str, versionlist[1:])), versionlist[0])
+        else:
+            result = (version, "")
+    finally:
+        file.close()
+    return result
 
 
 def version_numbers(version):
-    versionlist = version.split(".")
-    major = int(versionlist[0]) if (0 < len(versionlist)) else 1
-    minor = int(versionlist[1]) if (1 < len(versionlist)) else 0
-    if (2 < len(versionlist)):
-        updatelist = versionlist[2].split("-")
-        update = int(updatelist[0]) if (0 < len(updatelist)) else 0
-        patch = int(updatelist[1]) if (1 < len(updatelist)) else 0
-    else:
-        update = 0
-        patch = 0
+    versionlist = version.split("-")
+    if (1 < len(versionlist)): patch = int(versionlist[1])
+    else: patch = 0
+    if (0 < len(versionlist)): versionlist = versionlist[0].split(".")
+    else: versionlist = list()
+    if (0 < len(versionlist)): major = int(versionlist[0])
+    else: major = 1
+    if (1 < len(versionlist)): minor = int(versionlist[1])
+    else: minor = 0
+    if (2 < len(versionlist)): update = int(versionlist[2])
+    else: update = 0
     return major, minor, update, patch
 
 
@@ -152,7 +183,7 @@ if __name__ == "__main__":
         mnk_size = int(sys.argv[3])
         dims = load_mnklist(sys.argv[4:4+mnk_size], threshold, -1)
         dims = load_mnklist(sys.argv[4+mnk_size:], threshold, -2, dims)
-        print(" ".join(map(lambda mnk: "_".join(map(str, mnk)), dims)))
+        print(" ".join(map(lambda mnk: "_".join(map(str, mnk)), sorted(dims))))
     elif (4 == argc and 0 < arg1):
         print(align_value(arg1, int(sys.argv[2]), sanitize_alignment(int(sys.argv[3]))))
     else:
