@@ -78,11 +78,27 @@ typedef LIBXSMM_RETARGETABLE void (*libxsmm_smmfunction)(const float* a, const f
 /** Specialized function with fused alpha and beta arguments, and optional prefetch locations (double-precision). */
 typedef LIBXSMM_RETARGETABLE void (*libxsmm_dmmfunction)(const double* a, const double* b, double* c, ...);
 
+/** Specialized function with fused alpha and beta arguments, and optional prefetch locations (weak-typed). */
+typedef union LIBXSMM_RETARGETABLE libxsmm_xmmfunction {
+  libxsmm_smmfunction smm;
+  libxsmm_dmmfunction dmm;
+} libxsmm_xmmfunction;
+
 /** Initialize the library; pay for setup cost at a specific point. */
 LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_init(void);
 /** Uninitialize the library and free internal memory (optional). */
 LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_finalize(void);
 
+/**
+ * Returns the architecture and instruction set extension as determined by the CPUID flags.
+ * If 0 != LIBXSMM_JIT and LIBXSMM_X86_AVX <= result, then this instruction set extension
+ * is targeted by the JIT code generator.
+ */
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE int libxsmm_get_target_arch();
+
+/** Query or JIT-generate a function; return zero if it does not exist or if JIT is not supported (descriptor form). */
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE libxsmm_xmmfunction libxsmm_xmmdispatch(const libxsmm_gemm_descriptor* descriptor);
+
 /** Query or JIT-generate a function; return zero if it does not exist or if JIT is not supported (single-precision). */
 LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE libxsmm_smmfunction libxsmm_smmdispatch(int m, int n, int k,
   const int* lda, const int* ldb, const int* ldc,
@@ -103,14 +119,7 @@ LIBXSMM_INLINE_EXPORT LIBXSMM_RETARGETABLE void libxsmm_sgemm(const char* transa
 #if defined(LIBXSMM_BUILD)
 ;
 #else
-{ int flags = (0 != transa
-    ? (('N' == *transa || 'n' == *transa) ? (LIBXSMM_FLAGS & ~LIBXSMM_GEMM_FLAG_TRANS_A)
-                                          : (LIBXSMM_FLAGS |  LIBXSMM_GEMM_FLAG_TRANS_A))
-    : LIBXSMM_FLAGS);
-  flags = (0 != transb
-    ? (('N' == *transb || 'n' == *transb) ? (flags & ~LIBXSMM_GEMM_FLAG_TRANS_B)
-                                          : (flags |  LIBXSMM_GEMM_FLAG_TRANS_B))
-    : flags);
+{ LIBXSMM_GEMM_DECLARE_FLAGS(flags, transa, transb, m, n, k, a, b, c);
   LIBXSMM_SGEMM(flags, *m, *n, *k,
     0 != alpha ? *alpha : ((float)LIBXSMM_ALPHA),
     a, *(lda ? lda : LIBXSMM_LD(m, k)), b, *(ldb ? ldb : LIBXSMM_LD(k, n)),
@@ -128,14 +137,7 @@ LIBXSMM_INLINE_EXPORT LIBXSMM_RETARGETABLE void libxsmm_dgemm(const char* transa
 #if defined(LIBXSMM_BUILD)
 ;
 #else
-{ int flags = (0 != transa
-    ? (('N' == *transa || 'n' == *transa) ? (LIBXSMM_FLAGS & ~LIBXSMM_GEMM_FLAG_TRANS_A)
-                                          : (LIBXSMM_FLAGS |  LIBXSMM_GEMM_FLAG_TRANS_A))
-    : LIBXSMM_FLAGS);
-  flags = (0 != transb
-    ? (('N' == *transb || 'n' == *transb) ? (flags & ~LIBXSMM_GEMM_FLAG_TRANS_B)
-                                          : (flags |  LIBXSMM_GEMM_FLAG_TRANS_B))
-    : flags);
+{ LIBXSMM_GEMM_DECLARE_FLAGS(flags, transa, transb, m, n, k, a, b, c);
   LIBXSMM_DGEMM(flags, *m, *n, *k,
     0 != alpha ? *alpha : ((double)LIBXSMM_ALPHA),
     a, *(lda ? lda : LIBXSMM_LD(m, k)), b, *(ldb ? ldb : LIBXSMM_LD(k, n)),
@@ -144,6 +146,20 @@ LIBXSMM_INLINE_EXPORT LIBXSMM_RETARGETABLE void libxsmm_dgemm(const char* transa
 }
 #endif
 
+/** Threadable general dense matrix multiplication; requires linking libxsmmext (single-precision). */
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_omps_sgemm(const char* transa, const char* transb,
+  const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* k,
+  const float* alpha, const float* a, const libxsmm_blasint* lda,
+  const float* b, const libxsmm_blasint* ldb,
+  const float* beta, float* c, const libxsmm_blasint* ldc);
+
+/** Threadable general dense matrix multiplication; requires linking libxsmmext (double-precision). */
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_omps_dgemm(const char* transa, const char* transb,
+  const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* k,
+  const double* alpha, const double* a, const libxsmm_blasint* lda,
+  const double* b, const libxsmm_blasint* ldb,
+  const double* beta, double* c, const libxsmm_blasint* ldc);
+
 /** General dense matrix multiplication based on LAPACK/BLAS (single-precision). */
 LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_blas_sgemm(const char* transa, const char* transb,
   const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* k,
